package eu.maiora.routes

import eu.maiora.loadConfig
import eu.maiora.model.*
import io.ktor.http.*
import io.ktor.server.application.*
import io.ktor.server.auth.*
import io.ktor.server.auth.jwt.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlin.collections.ArrayList
import kotlin.math.round


fun Route.disponibilitaArticoli(articoliCrossQRicambiRepository: ArticoliCrossQRicambiRepository,
                                viewEcommerceArticoliRepository: ViewEcommerceArticoliRepository,
                                viewEcommercePrezSconDispRepository: ViewEcommercePrezSconDispRepository,
                                viewClientiAgentiRepository : ViewClientiAgentiRepository,
                                viewEcommerceValoreUsatoRepository: ViewEcommerceValoreUsatoRepository){
    route("/api/disponibilitaArticoli"){
        authenticate("auth-jwt") {
            get("{codiceArticolo}"){
                val username = call.authentication.principal<JWTPrincipal>()?.payload?.subject ?: ""

                // Ottieni il codice articolo dal percorso
                val codiceArticolo = call.parameters["codiceArticolo"]

                if (codiceArticolo == null) {
                    call.respondText("Codice articolo non valido", status = HttpStatusCode.BadRequest)
                    return@get
                }

                //ricerca del codice articolo
                //- verifico che il codice esista nel sistema interno, ovvero nella tabella ARTICOLI_CROSS_QRICAMBI
                //  la ricerca va effettuata prima per CODICE_ARTICOLO e poi per CODICE_CROSS
                //- se non esiste, restituisco not found
                //- altrimenti, recupero l'id dell'articolo dalla vista VIEW_ECOMMERCE_ARTICOLI
                //  e tutti i dati da mostrare (vedi GET_PREZ_SCON_DISP, named query in ViewEcommercePrezSconDisp)

                val properties = loadConfig()

                if(username.equals(properties.getProperty("qricambi.username").toString())){
                    val idClienteQRicambi = properties.getProperty("qricambi.id").toLong()
                    val listArticoliCrossQRicambi = articoliCrossQRicambiRepository.articoloByCodArt(codiceArticolo)
                    if(listArticoliCrossQRicambi.isNotEmpty()){
                        val articoloEcommerce = viewEcommerceArticoliRepository.articoloByCodice(listArticoliCrossQRicambi[0].codiceArticolo)
                        if(articoloEcommerce != null){
                            val dataValoreUsato = viewEcommerceValoreUsatoRepository.getValoreByIdArtIdClie(idClienteQRicambi, articoloEcommerce.id)
                            val data = viewEcommercePrezSconDispRepository.disponibilitaByIdArtIdClie(idClienteQRicambi, articoloEcommerce.id)
                            if(data != null){
                                var listNumPezziPerMagazzino = ArrayList<NumPezziPerMagazzino>()
                                var numPezziDisponibili = 0;
                                data.forEach { e ->
                                    numPezziDisponibili +=  e.numeroPezzi
                                }
                                listNumPezziPerMagazzino.add(NumPezziPerMagazzino("", numPezziDisponibili.toString()))

                                val result = DisponibilitaArticoli(articoloEcommerce.codice,
                                    articoloEcommerce.descrizione,
                                    listArticoliCrossQRicambi[0].costruttore,
                                    data[0].netto.toString(),
                                    data[0].numeroPezzi > 0,
                                    listNumPezziPerMagazzino,
                                    (dataValoreUsato?.firstOrNull()?.valoreUsato ?: "0.0").toString()
                                )
                                call.respond(result)
                            }
                            else {
                                call.respondText("Articolo ecommerce non trovato (prezzi sconti e disponibilita)", status = HttpStatusCode.NotFound)
                            }

                        }
                        else {
                            call.respondText("Articolo ecommerce non trovato", status = HttpStatusCode.NotFound)
                        }
                    }
                    else {
                        call.respondText("Articolo non trovato", status = HttpStatusCode.NotFound)
                    }
                }
                else {
                    //verificare il cliente collegato all'agente/username
                    val cliente = viewClientiAgentiRepository.clienteByAgente(username)
                    if(cliente != null){
                        val listArticoliCrossQRicambi = articoliCrossQRicambiRepository.articoloByCodArt(codiceArticolo)
                        if(!listArticoliCrossQRicambi.isEmpty()){
                            val articoloEcommerce = viewEcommerceArticoliRepository.articoloByCodice(listArticoliCrossQRicambi[0].codiceArticolo)
                            if(articoloEcommerce != null){
                                val dataValoreUsato = viewEcommerceValoreUsatoRepository.getValoreByIdArtIdClie(cliente.idCliente, articoloEcommerce.id)
                                val data = viewEcommercePrezSconDispRepository.disponibilitaByIdArtIdClie(cliente.idCliente, articoloEcommerce.id)
                                if(!data.isNullOrEmpty()){
                                    var listNumPezziPerMagazzino = ArrayList<NumPezziPerMagazzino>()
                                    data.forEach { e ->
                                        listNumPezziPerMagazzino.add(NumPezziPerMagazzino(e.magazzino, e.numeroPezzi.toString()))
                                    }

                                    //calcolo del netto:
                                    //se il prezzo  valorizzato, utilizzare gli sconti, altrimenti va restituito il valore del campo netto
                                    val netto = if(data[0].prezzo > 0)
                                        calcolaNetto(data[0].prezzo, data[0].sconto)
                                    else
                                        data[0].netto

                                    val result = DisponibilitaArticoli(articoloEcommerce.codice,
                                        articoloEcommerce.descrizione,
                                        listArticoliCrossQRicambi[0].costruttore,
                                        netto.toString(),
                                        data[0].numeroPezzi > 0,
                                        listNumPezziPerMagazzino,
                                        dataValoreUsato?.get(0)?.valoreUsato.toString()
                                    )
                                    call.respond(result)
                                }
                                else {
                                    call.respondText("Articolo ecommerce non trovato (prezzi sconti e disponibilita)", status = HttpStatusCode.NotFound)
                                }

                            }
                            else {
                                call.respondText("Articolo ecommerce non trovato", status = HttpStatusCode.NotFound)
                            }
                        }
                        else {
                            call.respondText("Articolo non trovato", status = HttpStatusCode.NotFound)
                        }
                    }
                    else {
                        call.respondText("Cliente collegato all'agente non trovato", status = HttpStatusCode.NotFound)
                    }
                }

            }
        }

    }
}

/***
 * calcolo del prezzo netto: utilizzare gli sconti solo se valorizzati
 */
fun calcolaNetto(prezzo : Double, sconto : Double) : Double {
    var netto = prezzo
    if(sconto > 0) {
        netto *= (1 - sconto / 100)
    }
    return (round(netto * 100) / 100.0)
}